%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Shows a current map and plots the eddy contours (curl)----------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% lon: array 1D longitudes
% lat: array 1D latitudes
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% rot_map: array 2D containing curl surface
% t: time
% depth: depth
% rotate_max_area: maximum area to check constraints
% max_angle: maximum angle allowed between eddy limit vectors
%
% automatic_info_eddy_array: array 1D containing eddy features
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function automatic_info_eddy_array = eddy_shape_curl_detection(lon,lat,U,V,t,depth,rotate_max_area,max_angle)
    automatic_info_eddy = struct('c_lon',-1,'c_lat',-1,'limit',[],'cyclonic',0);
    automatic_info_eddy_array = [];
    global visitados;

    hold off;
    draw_map(lon,lat,U,V,depth,t);
    axis equal;   
 
    [U,V]=delete_nan(U,V,t,depth);
    rot_map = curl(lon, lat, U(:,:,depth,t), V(:,:,depth,t));
    
    [real_centers_min,rot_centers_min,real_centers_max,rot_centers_max] = eddy_center_detection(U,V,rot_map,t,depth,rotate_max_area,max_angle);
    %Anticyclonic
    if(~isempty(real_centers_min))
        for i=1:length(rot_centers_min(1,:))
            value = get_boundary_value(rot_centers_min(1,i),rot_centers_min(2,i),rot_map,-1);
            visitados = [];
            limit = get_boundary_min(rot_centers_min(1,i),rot_centers_min(2,i),rot_map,value);
            correction_lon = real_centers_min(1,i) - rot_centers_min(1,i);
            correction_lat = real_centers_min(2,i) - rot_centers_min(2,i);
            for k=1:length(limit(1,:))
                limit(1,k) = lon(limit(1,k)+correction_lon);
                limit(2,k) = lat(limit(2,k)+correction_lat);
                plot(limit(1,k),limit(2,k),'r*');
            end
            limit = sort_eddy_contour(limit);
            if(length(limit(1,:)) > 1)
                automatic_info_eddy.c_lon = lon(real_centers_min(1,i));
                automatic_info_eddy.c_lat = lat(real_centers_min(2,i));
                automatic_info_eddy.limit = limit;
                automatic_info_eddy.cyclonic = 0;
                automatic_info_eddy_array = [automatic_info_eddy_array,automatic_info_eddy];
            end
        end
    end

    %Cyclonic
    if(~isempty(real_centers_max))
        for i=1:length(rot_centers_max(1,:))
            value = get_boundary_value(rot_centers_max(1,i),rot_centers_max(2,i),rot_map,1);
            visitados = [];
            limit = get_boundary_max(rot_centers_max(1,i),rot_centers_max(2,i),rot_map,value);
            correction_lon = real_centers_max(1,i) - rot_centers_max(1,i);
            correction_lat = real_centers_max(2,i) - rot_centers_max(2,i);
            for k=1:length(limit(1,:))
                limit(1,k) = lon(limit(1,k)+correction_lon);
                limit(2,k) = lat(limit(2,k)+correction_lat);
                plot(limit(1,k),limit(2,k),'b*');
            end
            limit = sort_eddy_contour(limit);
            if(length(limit(1,:)) > 1)
                automatic_info_eddy.c_lon = lon(real_centers_max(1,i));
                automatic_info_eddy.c_lat = lat(real_centers_max(2,i));
                automatic_info_eddy.limit = limit;
                automatic_info_eddy.cyclonic = 1;
                automatic_info_eddy_array = [automatic_info_eddy_array,automatic_info_eddy];
            end
        end
    end
    save(strcat(strcat(strcat(strcat('automatic_curl_',num2str(depth)),'_'),num2str(t)),'.mat'),'automatic_info_eddy_array');
end


function angle = first_square(angle)
    angle = simplify_angle(angle);
    if(angle >= 3*pi/2)
        angle = angle - 3*pi/2;
    else
        if(angle >= pi)
            angle = angle - pi;
        else
            if(angle >= pi/2)
                angle = angle - pi/2;
            end
        end
    end
end

function limit = get_boundary_min(lon,lat,rot_map,value)
    global visitados;
    limit_1 = [];
    limit_2 = [];
    limit_3 = [];
    limit_4 = [];
    lon_size = length(rot_map(1,:));
    lat_size = length(rot_map(:,1));
    visitados = [visitados,[lon;lat]];
    if((lat>1) && (lon>1) && (lon < lon_size) && (lat < lat_size) && (rot_map(lat,lon) < 0) && (rot_map(lat,lon) < value))
        if(isempty(find(ismember(visitados',[lon-1,lat],'rows'),1)))
            limit_1 = get_boundary_min(lon-1,lat,rot_map,value);
        end
        if(isempty(find(ismember(visitados',[lon+1,lat],'rows'),1)))
            limit_2 = get_boundary_min(lon+1,lat,rot_map,value);
        end
        if(isempty(find(ismember(visitados',[lon,lat-1],'rows'),1)))
            limit_3= get_boundary_min(lon,lat-1,rot_map,value);
        end
        if(isempty(find(ismember(visitados',[lon,lat+1],'rows'),1)))                                               
            limit_4 = get_boundary_min(lon,lat+1,rot_map,value);
        end
        limit = [limit_1,limit_2,limit_3,limit_4];
    else
        limit(1,1) = lon;
        limit(2,1) = lat;
    end
end


function limit = get_boundary_max(lon,lat,rot_map,value)
    global visitados;
    limit_1 = [];
    limit_2 = [];
    limit_3 = [];
    limit_4 = [];
    lon_size = length(rot_map(1,:));
    lat_size = length(rot_map(:,1));
    visitados = [visitados,[lon;lat]];
    if((lat>1) && (lon>1) && (lon < lon_size) && (lat < lat_size) && (rot_map(lat,lon) > 0) && (rot_map(lat,lon) > value))
        if(isempty(find(ismember(visitados',[lon-1,lat],'rows'),1)))
            limit_1 = get_boundary_max(lon-1,lat,rot_map,value);
        end
        if(isempty(find(ismember(visitados',[lon+1,lat],'rows'),1)))
            limit_2 = get_boundary_max(lon+1,lat,rot_map,value);
        end
        if(isempty(find(ismember(visitados',[lon,lat-1],'rows'),1)))
            limit_3= get_boundary_max(lon,lat-1,rot_map,value);
        end
        if(isempty(find(ismember(visitados',[lon,lat+1],'rows'),1)))                                               
            limit_4 = get_boundary_max(lon,lat+1,rot_map,value);
        end
        limit = [limit_1,limit_2,limit_3,limit_4];
    else
        limit(1,1) = lon;
        limit(2,1) = lat;
    end
end


function value = get_boundary_value(c_lon,c_lat,rot_map,max_min)
    step = [[-1;0],[-1;1],[0;1],[1;1],[1;0],[1;-1],[0;-1],[-1;-1]];
    min = 10000;
    max = -10000;
    for i=1:8
        lon = c_lon + 3*step(1,i);
        lat = c_lat + 3*step(2,i);
        while((floor(lon) > 0) && (floor(lon) <= length(rot_map(1,:))) && (floor(lat) > 0) && (floor(lat) <= length(rot_map(:,1))) && abs(rot_map(floor(lat),floor(lon)) - rot_map(floor(lat-step(2,i)),floor(lon-step(1,i)))) >= abs(rot_map(floor(lat-step(2,i)),floor(lon-step(1,i))) - rot_map(floor(lat-2*step(2,i)),floor(lon-2*step(1,i)))))
            lon = lon + step(1,i);
            lat = lat +  step(2,i);
        end
        if((lon > 0) && (lon <= length(rot_map(1,:))) && (lat > 0) && (lat <= length(rot_map(:,1))))
            if(max_min == 1)
                if(rot_map(floor(lat),floor(lon)) > max)
                    max = rot_map(floor(lat),floor(lon));
                end
            else
                if(rot_map(floor(lat),floor(lon)) < min)
                    min = rot_map(floor(lat),floor(lon));
                end
            end
        end
    end
    if(max_min == 1)
        value = max;
    else
        value = min;
    end
end