%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Shows a current map and plots the eddy life-time ---------------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% lon: array 1D containing longitudes
% lat: array 1D containing latitudes
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% t_min: time to start tracking
% t_max: time to finish tracking
% rotate_max_area: maximum area to check constraints
% max_angle: maximum angle (radians) allowed between current vectors
%
% trace_min: anticyclonic trace
% trace_max: cyclonic trace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [trace_min,trace_max] = eddy_track(lon,lat,U,V,t_min,t_max,depth,rotate_max_area,max_angle_diff)
    hold off;
    draw_map(lon,lat,U,V,depth,t_min);
    axis equal;
    [U,V]=delete_nan(U,V,t_min,depth);
    rot_map = curl(lon, lat, U(:,:,depth,t_min), V(:,:,depth,t_min));
    [real_centers_min,~,real_centers_max,~] = eddy_center_detection(U,V,rot_map,t_min,depth,rotate_max_area,max_angle_diff);
    trace_min = zeros(2,length(real_centers_min),t_max-t_min+1);
    trace_max = zeros(2,length(real_centers_max),t_max-t_min+1);
    if(~isempty(real_centers_min))
        for i=1:length(real_centers_min(1,:))
            actual_lon = real_centers_min(1,i);
            actual_lat = real_centers_min(2,i);
            iterator = 1;
            for j=t_min:t_max
                if(j==t_min)
                    plot(lon(actual_lon),lat(actual_lat),'ro');
                    trace_min(1,i,iterator) = actual_lon;
                    trace_min(2,i,iterator) = actual_lat;
                else
                    [~,curr_lon,curr_lat] = get_current_min(actual_lon,actual_lat,U,V,j,depth);
                    [res,type] = check_center_constraints(U,V,curr_lon,curr_lat,j,depth,rotate_max_area,max_angle_diff);
                    if((res == 1) && (type == 1))
                        trace_min(1,i,iterator) = curr_lon;
                        trace_min(2,i,iterator) = curr_lat;
                    end
                    actual_lon = curr_lon;
                    actual_lat = curr_lat;
                end
                iterator = iterator + 1;
            end
        end
    end 
    if(~isempty(real_centers_max))
        for i=1:length(real_centers_max(1,:))
            actual_lon = real_centers_max(1,i);
            actual_lat = real_centers_max(2,i);
            iterator = 1;
            for j=t_min:t_max
                if(j==t_min)
                    plot(lon(actual_lon),lat(actual_lat),'bo');
                    trace_max(1,i,iterator) = actual_lon;
                    trace_max(2,i,iterator) = actual_lat;
                else
                    [~,curr_lon,curr_lat] = get_current_min(actual_lon,actual_lat,U,V,j,depth);
                    [res,type] = check_center_constraints(U,V,curr_lon,curr_lat,j,depth,rotate_max_area,max_angle_diff);
                    if((res == 1) && (type == 2))
                        trace_max(1,i,iterator) = curr_lon;
                        trace_max(2,i,iterator) = curr_lat;                
                    end
                    actual_lon = curr_lon;
                    actual_lat = curr_lat;
                end
                iterator = iterator + 1;
            end
        end
    end
     
    if(~isempty(real_centers_min))
        for i=1:length(real_centers_min(1,:))
            iterator = 1;
            for j=t_min:t_max
                if((trace_min(1,i,iterator) == 0) || (trace_min(2,i,iterator) == 0))
                    trace_min(1,i,iterator) = trace_min(1,i,iterator-1);
                    trace_min(2,i,iterator) = trace_min(2,i,iterator-1);
                    plot(lon(trace_min(1,i,iterator)),lat(trace_min(2,i,iterator)),'r*','linewidth',6);
                end
                iterator = iterator + 1;
            end
            plot(lon(trace_min(1,i,iterator-1)),lat(trace_min(2,i,iterator-1)),'r*','linewidth',6);
        end
    end
    if(~isempty(real_centers_max))
        for i=1:length(real_centers_max(1,:))
            iterator = 1;
            for j=t_min:t_max
                if((trace_max(1,i,iterator) == 0) || (trace_max(2,i,iterator) == 0))
                    trace_max(1,i,iterator) = trace_max(1,i,iterator-1);
                    trace_max(2,i,iterator) = trace_max(2,i,iterator-1);
                    plot(lon(trace_max(1,i,iterator)),lat(trace_max(2,i,iterator)),'b*','linewidth',6);
                end
                iterator = iterator + 1;
            end
            plot(lon(trace_max(1,i,iterator-1)),lat(trace_max(2,i,iterator-1)),'b*','linewidth',6);
        end
    end
     
    if(~isempty(real_centers_min))
        for i=1:length(real_centers_min(1,:))
            plot(lon(squeeze(trace_min(1,i,:))),lat(squeeze(trace_min(2,i,:))),'r','linewidth',2);
        end
    end
    if(~isempty(real_centers_max))
        for i=1:length(real_centers_max(1,:))
            plot(lon(squeeze(trace_max(1,i,:))),lat(squeeze(trace_max(2,i,:))),'b','linewidth',2);
        end
    end
end