function info_eddy = estimate_3D_eddy(info_eddy,max_depth,lon,lat,depth,r1,r2,angle,speed)
    if(length(speed(:,1)) == 1)
        speed = speed';
    end
    [up_depth,down_depth] = get_up_down_depths(info_eddy.modified_depths,max_depth,depth);
    if((up_depth ~= -1) || (down_depth ~= -1))
        modified_eddies = get_modified_eddies(info_eddy,up_depth,down_depth,lon,lat,depth,r1,r2,angle,speed);
        info_eddy = change_info_eddy(info_eddy,modified_eddies,up_depth,down_depth,depth);
    end
end


function [up_depth,down_depth] = get_up_down_depths(modified_depths,max_depth,depth)
    up_depth = -1;
    down_depth = -1;
    for i=depth-1:-1:1
        if(modified_depths(i))
            up_depth = i;
            break;
        end
    end
    for i=depth+1:max_depth
        if(modified_depths(i))
            down_depth = i;
            break;
        end
    end
end


function modified_eddies = get_modified_eddies(info_eddy,up_depth,down_depth,lon,lat,depth,r1,r2,angle,speed)
    vec_lon_1 = [];
    vec_lat_1 = [];
    vec_r1_1 = [];
    vec_r2_1 = [];
    vec_angle_1 = [];
    if(up_depth ~= -1)
        cub_speed_1 = zeros(abs(up_depth-depth)-1,length(info_eddy.speed(up_depth,:,1)),length(info_eddy.speed(up_depth,1,:)));
        speed_up = abs(up_depth-depth)-1;
    else
        cub_speed_1 = [];
        speed_up = 0;
    end
    
    vec_lon_2 = [];
    vec_lat_2 = [];
    vec_r1_2 = [];
    vec_r2_2 = [];
    vec_angle_2 = [];
    if(down_depth ~= -1)
        cub_speed_2 = zeros(abs(down_depth-depth)-1,length(info_eddy.speed(down_depth,:,1)),length(info_eddy.speed(down_depth,1,:)));
        speed_down = abs(down_depth-depth)-1;
    else
        speed_down = 0;
        cub_speed_2 = [];
    end
    
    if(up_depth ~= -1)
        vec_lon_1 = linear_interpolation(info_eddy.c_lon(up_depth),lon,depth-up_depth,1);
        vec_lat_1 = linear_interpolation(info_eddy.c_lat(up_depth),lat,depth-up_depth,1);
        vec_r1_1 = linear_interpolation(info_eddy.r1(up_depth),r1,depth-up_depth,1);
        vec_r2_1 = linear_interpolation(info_eddy.r2(up_depth),r2,depth-up_depth,1);
        vec_angle_1 = linear_interpolation(info_eddy.angle(up_depth),angle,depth-up_depth,1);
        for i=1:length(info_eddy.speed(up_depth,:,1))
            for j=1:length(info_eddy.speed(up_depth,1,:))
                vec = linear_interpolation(info_eddy.speed(up_depth,i,j),speed(i,j),depth-up_depth,1);
                cub_speed_1(:,i,j) = linear_interpolation(info_eddy.speed(up_depth,i,j),speed(i,j),depth-up_depth,1);
            end
        end                
    end
    
    if(down_depth ~= -1)
        vec_lon_2 = linear_interpolation(info_eddy.c_lon(down_depth),lon,down_depth-depth,0);
        vec_lat_2 = linear_interpolation(info_eddy.c_lat(down_depth),lat,down_depth-depth,0);
        vec_r1_2 = linear_interpolation(info_eddy.r1(down_depth),r1,down_depth-depth,0);
        vec_r2_2 = linear_interpolation(info_eddy.r2(down_depth),r2,down_depth-depth,0);
        vec_angle_2 = linear_interpolation(info_eddy.angle(down_depth),angle,down_depth-depth,0);
        for i=1:length(info_eddy.speed(down_depth,:,1))
            for j=1:length(info_eddy.speed(down_depth,1,:))
                val = linear_interpolation(info_eddy.speed(down_depth,i,j),speed(i,j),down_depth-depth,0);
                cub_speed_2(:,i,j) = linear_interpolation(info_eddy.speed(down_depth,i,j),speed(i,j),down_depth-depth,0);
            end
        end   
    end
    
    modified_eddies.c_lon = [vec_lon_1,lon,flipdim(vec_lon_2,2)];
    modified_eddies.c_lat = [vec_lat_1,lat,flipdim(vec_lat_2,2)];
    modified_eddies.r1 = [vec_r1_1,r1,flipdim(vec_r1_2,2)];
    modified_eddies.r2 = [vec_r2_1,r2,flipdim(vec_r2_2,2)];
    modified_eddies.angle = [vec_angle_1,angle,flipdim(vec_angle_2,2)];
    
    modified_eddies.speed = zeros(speed_up+speed_down+1,length(speed(:,1)),length(speed(1,:)));
    
    iterator = 1;
    actual = 0;
    cub_speed_2 = flipdim(cub_speed_2,1);
    for i=1:length(modified_eddies.speed(:,1,1))
        if(i <= speed_up)
            for j=1:length(speed(:,1))
                for k=1:length(speed(1,:))
                    modified_eddies.speed(i,j,k) = cub_speed_1(i,j,k);
                end
            end
        else
            if(actual == 0)
                actual = 1;
                for j=1:length(speed(:,1))
                    for k=1:length(speed(1,:))
                        modified_eddies.speed(i,j,k) = speed(j,k);
                    end
                end
            else
                for j=1:length(speed(:,1))
                    for k=1:length(speed(1,:))
                        modified_eddies.speed(i,j,k) = cub_speed_2(iterator,j,k);
                    end
                end
                iterator = iterator + 1;
            end;
        end
    end
end


function vector = linear_interpolation(value_1,value_2,interval,up_down)
    vector = zeros(1,interval-1);
    if(up_down == 1)
        m = (value_1-value_2)/interval;
        for i=1:interval-1
            vector(i) = m*i+value_2;
        end
        vector = flipdim(vector,2);
    else
        m = (value_2-value_1)/interval;
        for i=1:interval-1
            vector(i)= m*i+value_1;
        end
    end
    
end

function info_eddy = change_info_eddy(info_eddy,modified_eddies,up_depth,down_depth,depth)
    iterator = 1;
    if(up_depth == -1)
        up_depth = depth-1;
    end
    if(down_depth == -1)
        down_depth = depth+1;
    end
    for i=up_depth+1:down_depth-1
        info_eddy.c_lon(i) = modified_eddies.c_lon(iterator);
        info_eddy.c_lat(i) = modified_eddies.c_lat(iterator);
        info_eddy.r1(i) = modified_eddies.r1(iterator);
        info_eddy.r2(i) = modified_eddies.r2(iterator);
        info_eddy.angle(i) = modified_eddies.angle(iterator);
        info_eddy.speed(i,:,:) = modified_eddies.speed(iterator,:,:);
        iterator = iterator + 1;
    end
end