%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Estimates eddy size----------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% axis_1: array 1D containing axis 1 coordinates (lon_1,lat_1,lon_2,lat_2) (matrix coordinate system)
% axis_2: array 1D containing axis 2 coordinates (lon_3,lat_3,lon_4,lat_4) (matrix coordinate system)
% angle: eddy angle (ellipse angle)
% lon: eddy center longitude (matrix coordinate system)
% lat: eddy center latitude (matrix coordinate system)
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% t: time
% depth: depth
% cyclonic: 1 if cyclonic, 0 if not
%
% axis_1: array 1D containing new axis 1 coordinates (lon_1,lat_1,lon_2,lat_2) (matrix coordinate system)
% axis_2: array 1D containing new axis 2 coordinates (lon_3,lat_3,lon_4,lat_4) (matrix coordinate system)
% error: result error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [axis_1,axis_2,error] = estimate_eddy_size(axis_1,axis_2,angle,lon,lat,U,V,t,depth,cyclonic)
    [axis_1,axis_2] = get_biggest_eddy(axis_1,axis_2,angle,lon,lat,U,V,t,depth);
    [axis_1,axis_2,error] = estimate_eddy_shape(axis_1,axis_2,angle,lon,lat,U,V,t,depth,cyclonic);
end


function [axis_1,axis_2] = get_biggest_eddy(axis_1,axis_2,angle,lon,lat,U,V,t,depth)
    [axis_1,axis_2,new_axis_1,new_axis_2] = get_first_too_big_eddy_size(axis_1,axis_2,angle,lon,lat,U,V,t,depth);
    [axis_1,axis_2] = get_first_correct_eddy_size(axis_1,axis_2,new_axis_1,new_axis_2,angle,lon,lat,U,V,t,depth);
end


function [axis_1,axis_2,new_axis_1,new_axis_2] = get_first_too_big_eddy_size(axis_1,axis_2,angle,lon,lat,U,V,t,depth)
    new_axis_1 = axis_1;
    new_axis_2 = axis_2;
    first = 1;
    while((first == 1) || (check_center_constraints_by_axis(new_axis_1,new_axis_2,angle,lon,lat,U,V,t,depth)))
        first = 0;
        axis_1 = new_axis_1;
        axis_2 = new_axis_2;
        [new_axis_1,new_axis_2] = resize_axis(lon,lat,axis_1,axis_2,0,0,1);
    end
end


function [new_axis_1,new_axis_2] = resize_axis(lon,lat,axis_1,axis_2,old_axis_1,old_axis_2,expand)
    dist_lon_1 = abs(lon - axis_1(1));
    dist_lon_2 = abs(lon - axis_2(1));
    dist_lat_1 = abs(lat - axis_1(2));
    dist_lat_2 = abs(lat - axis_2(2));   
    
    if(expand)
        if(axis_1(1) < lon) new_axis_1(1) = axis_1(1) - dist_lon_1; else new_axis_1(1) = axis_1(1) + dist_lon_1; end
        if(axis_1(3) < lon) new_axis_1(3) = axis_1(3) - dist_lon_1; else new_axis_1(3) = axis_1(3) + dist_lon_1; end
        if(axis_1(2) < lat) new_axis_1(2) = axis_1(2) - dist_lat_1; else new_axis_1(2) = axis_1(2) + dist_lat_1; end
        if(axis_1(4) < lat) new_axis_1(4) = axis_1(4) - dist_lat_1; else new_axis_1(4) = axis_1(4) + dist_lat_1; end

        if(axis_2(1) < lon) new_axis_2(1) = axis_2(1) - dist_lon_2; else new_axis_2(1) = axis_2(1) + dist_lon_2; end
        if(axis_2(3) < lon) new_axis_2(3) = axis_2(3) - dist_lon_2; else new_axis_2(3) = axis_2(3) + dist_lon_2; end
        if(axis_2(2) < lat) new_axis_2(2) = axis_2(2) - dist_lat_2; else new_axis_2(2) = axis_2(2) + dist_lat_2; end
        if(axis_2(4) < lat) new_axis_2(4) = axis_2(4) - dist_lat_2; else new_axis_2(4) = axis_2(4) + dist_lat_2; end
    else
        if(axis_1(1) < lon) new_axis_1(1) = axis_1(1) + abs(axis_1(1)-old_axis_1(1))/2; else new_axis_1(1) = axis_1(1) - abs(axis_1(1)-old_axis_1(1))/2; end
        if(axis_1(3) < lon) new_axis_1(3) = axis_1(3) + abs(axis_1(3)-old_axis_1(3))/2; else new_axis_1(3) = axis_1(3) - abs(axis_1(3)-old_axis_1(3))/2; end
        if(axis_1(2) < lat) new_axis_1(2) = axis_1(2) + abs(axis_1(2)-old_axis_1(2))/2; else new_axis_1(2) = axis_1(2) - abs(axis_1(2)-old_axis_1(2))/2; end
        if(axis_1(4) < lat) new_axis_1(4) = axis_1(4) + abs(axis_1(4)-old_axis_1(4))/2; else new_axis_1(4) = axis_1(4) - abs(axis_1(4)-old_axis_1(4))/2; end

        if(axis_2(1) < lon) new_axis_2(1) = axis_2(1) + abs(axis_2(1)-old_axis_2(1))/2; else new_axis_2(1) = axis_2(1) - abs(axis_2(1)-old_axis_2(1))/2; end
        if(axis_2(3) < lon) new_axis_2(3) = axis_2(3) + abs(axis_2(3)-old_axis_2(3))/2; else new_axis_2(3) = axis_2(3) - abs(axis_2(3)-old_axis_2(3))/2; end
        if(axis_2(2) < lat) new_axis_2(2) = axis_2(2) + abs(axis_2(2)-old_axis_2(2))/2; else new_axis_2(2) = axis_2(2) - abs(axis_2(2)-old_axis_2(2))/2; end
        if(axis_2(4) < lat) new_axis_2(4) = axis_2(4) + abs(axis_2(4)-old_axis_2(4))/2; else new_axis_2(4) = axis_2(4) - abs(axis_2(4)-old_axis_2(4))/2; end
    end
end


function [axis_1,axis_2] = get_first_correct_eddy_size(old_axis_1,old_axis_2,axis_1,axis_2,angle,lon,lat,U,V,t,depth)
    MAX_ITERATIONS = 5;
    iterations = 0;
    while((iterations < MAX_ITERATIONS) && (~check_center_constraints_by_axis(axis_1,axis_2,angle,lon,lat,U,V,t,depth)))
        [axis_1,axis_2] = resize_axis(lon,lat,axis_1,axis_2,old_axis_1,old_axis_2,0);
        iterations = iterations + 1;
    end
    if(iterations == 5)
        axis_1 = old_axis_1;
        axis_2 = old_axis_2;
    end
end