%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% - Gets local minimum current position----------------%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% @parameters
%
% lon: starting longitude (matrix coordinate system)
% lat: starting latitude (matrix coordinate system)
% U: array 4D (lon,lat,depth,time) containing U current component
% V: array 4D (lon,lat,depth,time) containing V current component
% t: time
% depth: depth
%
% val: result current magnitud
% index_lon: result longitud
% index_lat: result latitude
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [val,index_lon,index_lat] = get_current_min(lon,lat,U,V,t,depth)

    lon_size = length(U(1,:,1,1));
    lat_size = length(U(:,1,1,1));
  
    north=10000;
    south=10000;
    east=10000;
    west=10000;
    northEast=10000;
    northWest=10000;
    southEast=10000;
    southWest=10000;
    
    actual = sqrt(U(lat,lon,depth,t)^2+V(lat,lon,depth,t)^2);
    if(inRange(lon,lat+1,lon_size,lat_size)) north = sqrt(U(lat+1,lon,depth,t)^2+V(lat+1,lon,depth,t)^2); end
    if(inRange(lon,lat-1,lon_size,lat_size))south = sqrt(U(lat-1,lon,depth,t)^2+V(lat-1,lon,depth,t)^2); end
    if(inRange(lon+1,lat,lon_size,lat_size)) east = sqrt(U(lat,lon+1,depth,t)^2+V(lat,lon+1,depth,t)^2); end
    if(inRange(lon-1,lat,lon_size,lat_size)) west = sqrt(U(lat,lon-1,depth,t)^2+V(lat,lon-1,depth,t)^2); end
    if(inRange(lon+1,lat+1,lon_size,lat_size)) northEast = sqrt(U(lat+1,lon+1,depth,t)^2+V(lat+1,lon+1,depth,t)^2); end 
    if(inRange(lon-1,lat+1,lon_size,lat_size)) northWest = sqrt(U(lat+1,lon-1,depth,t)^2+V(lat+1,lon-1,depth,t)^2); end
    if(inRange(lon+1,lat-1,lon_size,lat_size)) southEast = sqrt(U(lat-1,lon+1,depth,t)^2+V(lat-1,lon+1,depth,t)^2); end
    if(inRange(lon-1,lat-1,lon_size,lat_size)) southWest = sqrt(U(lat-1,lon-1,depth,t)^2+V(lat-1,lon-1,depth,t)^2); end
    
    curr_vec = [actual,north,south,east,west,northEast,northWest,southEast,southWest];
    index_vec = [0 0; 0 1; 0 -1; 1 0; -1 0; 1 1; -1 1; 1 -1; -1 -1];
    
    [val,index] = min(curr_vec);
    if(val == actual)
        [new_val,new_lon,new_lat] = expand_search(lon,lat,U,V,t,depth,lon_size,lat_size);
        if(val == new_val)
            index_lon = lon;
            index_lat = lat;
        else
            [val,index_lon,index_lat] = get_current_min(new_lon,new_lat,U,V,t,depth);
        end
    else
        [val,index_lon,index_lat] = get_current_min(lon+index_vec(index,1),lat+index_vec(index,2),U,V,t,depth);
    end
end

function [min,new_lon,new_lat] = expand_search(lon,lat,U,V,t,depth,lon_size,lat_size)
    min = 10000;
    for i=-2:2
        for j=-2:2
            if(inRange(lon+j,lat+i,lon_size,lat_size))
                temp = sqrt(U(lat+i,lon+j,depth,t)^2+V(lat+i,lon+j,depth,t)^2);
                if(temp < min)
                    min = temp;
                    new_lat = lat+i;
                    new_lon = lon+j;
                end
            end
        end
    end
end

            
function B = inRange(lon,lat,lon_size,lat_size)
    if((lon < lon_size) && (lon > 0) && (lat < lat_size) && (lat > 0))
        B = 1;
    else 
        B = 0;
    end
end